import numpy as np

class PCA:
    def __init__(self):
        self.loadings = 0
        self.scores = 0
        self.singular_values = 0
        self.expl_var_ratio = 0

    #Calculates the PCA of a matrix with each column representing a variable, and the samples represented by the rows.
    def calculate_pca(self, matrix):
        cov = np.cov(matrix.T)
        self.singular_values, self.loadings = np.linalg.eig(cov)
        index = self.singular_values.argsort()[::-1] #make sure the singular values are in decending order as well as the loadings
        self.singular_values = self.singular_values[index]
        self.loadings = self.loadings[:,index] #sort the loadings the same way as the singular values
        self.expl_var_ratio = self.singular_values/np.sum(self.singular_values) #calculate the percentage of explained variance for each of the eigen values - this corresponds to the contribution of each PC
        self.scores = matrix @ self.loadings

    def apply_pca(self, matrix):
        return matrix @ self.loadings



import cv2 as cv
import numpy as np
import os

def import_data_cube(path):
    """
    ########## HSTI_import ##########
    This function takes an HSTI numpy array and exports it as individual .ppm
    images to a folder given by folder_name.
    """

    impath = path + '/images/capture/'

    try:
        number_of_image_files = sum([ '.ppm' in s for s in os.listdir(impath)])

        # steps = np.linspace(0,number_of_image_files*10-10,number_of_image_files)
        # imgs = np.zeros((768,1024,len(steps)))

        # for idx,i in enumerate(steps):
        #     imgs[:,:,idx] = cv.imread(impath+'step'+str(int(i)) + '.ppm',cv.IMREAD_ANYDEPTH)
        # imgs = np.rot90(imgs,1)
        #
        # print('Hyperspectral image shape:')
        # print(imgs.shape)

        steps = np.arange(0,number_of_image_files*10,10)
        imgs = []
        for step in steps:
            imgs.append(np.rot90(cv.imread(f'{impath}step{step}.ppm',cv.IMREAD_ANYDEPTH)))
        imgs = np.array(imgs)
        imgs = np.moveaxis(imgs, 0, 2)
        print('Hyperspectral image shape:')
        print(imgs.shape)

    except:
        print('Path should be directory containing the \'images\' directory.')

    return imgs

def import_image_acquisition_settings(path):

    f = open(path + "/output.txt", "r")
    i = 0
    line = []
    for x in f:
        if len(x.split(' ')) > 10:
            line = i
            break
        i += 1

    values  = np.loadtxt(path + '/output.txt',skiprows=line)

    sens_T  = np.round((np.mean(values[:,7])/100+np.mean(values[:,8])/100+np.mean(values[:,9])/100)/3,2)
    GSK     = np.mean(values[:,10])
    GFID    = np.mean(values[:,11])
    Gain    = np.mean(values[:,12])

    print('Sensor temperature: '+ str(sens_T))
    print('GSK: ' + str(int(GSK)))
    print('GFID: ' + str(int(GFID)))
    print('Gain: ' + str(Gain))

    valdict = {
      'SENS_T': sens_T,
      'GSK': GSK,
      'GFID': GFID,
      'GAIN': Gain
    }

    return valdict



import numpy as np
import os

def form_mirror_sep_axis(path):

    impath = path + 'images/capture/'
    number_of_bands = sum([ '.ppm' in s for s in os.listdir(impath)])
    t = np.arange(0,number_of_bands,1)

    p = np.poly1d(np.load(path + 'mirror_sep.npy'))
    mirror_sep = p(t)
    return mirror_sep*1e-6

def form_mirror_sep_axis_direct(path, number_of_bands):

    t = np.arange(0,number_of_bands,1)

    p = np.poly1d(np.load(path))
    mirror_sep = p(t)
    return mirror_sep*1e-6
